@@ -30,10 +30,11 @@
 #include "eel-lib-self-check-functions.h"
 #include "eel-enumeration.h"
 #include "eel-glib-extensions.h"
+#include "eel-string-list.h"
 #include "eel-string.h"
 #include <gconf/gconf-client.h>
 #include <gconf/gconf.h>
-#include <gtk/gtk.h>
+#include <gtk/gtksignal.h>
 
 /* An enumeration used for updating auto-storage variables in a type-specific way. 
  * FIXME: there is another enumeration like this in eel-global-preferences.c,
@@ -43,8 +44,8 @@ typedef enum {
 	PREFERENCE_BOOLEAN = 1,
 	PREFERENCE_INTEGER,
 	PREFERENCE_STRING,
-	PREFERENCE_STRING_ARRAY,
-	PREFERENCE_STRING_ARRAY_AS_QUARKS
+	PREFERENCE_STRING_LIST,
+	PREFERENCE_STRING_GLIST
 } PreferenceType;
 
 /*
@@ -94,7 +95,7 @@ preferences_gconf_value_get_int (const GConfValue *value)
 	if (value == NULL) {
 		return 0;
 	}
-	g_assert (value->type == GCONF_VALUE_INT);
+	g_return_val_if_fail (value->type == GCONF_VALUE_INT, 0);
 	return gconf_value_get_int (value);
 }
 
@@ -104,7 +105,7 @@ preferences_gconf_value_get_bool (const GConfValue *value)
 	if (value == NULL) {
 		return FALSE;
 	}
-	g_assert (value->type == GCONF_VALUE_BOOL);
+	g_return_val_if_fail (value->type == GCONF_VALUE_BOOL, FALSE);
 	return gconf_value_get_bool (value);
 }
 
@@ -114,39 +115,32 @@ preferences_gconf_value_get_string (const GConfValue *value)
 	if (value == NULL) {
 		return g_strdup ("");
 	}
-	g_assert (value->type == GCONF_VALUE_STRING);
+	g_return_val_if_fail (value->type == GCONF_VALUE_STRING, g_strdup (""));
 	return g_strdup (gconf_value_get_string (value));
 }
 
-static char **
-preferences_gconf_value_get_string_array (const GConfValue *value)
+static EelStringList *
+preferences_gconf_value_get_string_list (const GConfValue *value)
 {
-	GSList *slist, *l;
-	GPtrArray *result;
+	GSList *slist;
+	EelStringList *result;
 
 	if (value == NULL) {
-		return NULL;
+		return eel_string_list_new (TRUE);
 	}
-
-	g_assert (value->type == GCONF_VALUE_LIST);
-	g_assert (gconf_value_get_list_type (value) == GCONF_VALUE_STRING);
+	g_return_val_if_fail (value->type == GCONF_VALUE_LIST, eel_string_list_new (TRUE));
+	g_return_val_if_fail (gconf_value_get_list_type (value) == GCONF_VALUE_STRING, eel_string_list_new (TRUE));
 
 	slist = eel_gconf_value_get_string_list (value);
-
-	result = g_ptr_array_new ();
-	for (l = slist; l != NULL; l = l->next) {
-		g_ptr_array_add (result, l->data);
-	}
-	g_slist_free (slist);
-	g_ptr_array_add (result, NULL);
-
- 	return (char **) g_ptr_array_free (result, FALSE);
+	result = eel_string_list_new_from_g_slist (slist, TRUE);
+	eel_g_slist_free_deep (slist);
+ 	return result;
 }
 
 static const char *
 preferences_peek_storage_path (void)
 {
-	g_assert (storage_path != NULL);
+	g_return_val_if_fail (storage_path != NULL, NULL);
 
 	return storage_path;
 }
@@ -154,7 +148,7 @@ preferences_peek_storage_path (void)
 static void
 preferences_set_storage_path (const char *new_storage_path)
 {
-	g_assert (eel_strlen (new_storage_path) > 0);
+	g_return_if_fail (eel_strlen (new_storage_path) > 0);
 
 	/* Make sure the path is indeed different */
 	if (eel_str_is_equal (new_storage_path, storage_path)) {
@@ -187,8 +181,8 @@ preferences_get_value (const char *name)
 	char *key;
 	PreferencesEntry *entry;
 	
-	g_assert (name != NULL);
-	g_assert (preferences_is_initialized ());
+	g_return_val_if_fail (name != NULL, NULL);
+	g_return_val_if_fail (preferences_is_initialized (), NULL);
 
 	key = preferences_key_make (name);
 	result = eel_gconf_get_value (key);
@@ -209,7 +203,7 @@ preferences_get_value (const char *name)
 static gboolean
 preferences_preference_is_gconf_key (const char *name)
 {
-	g_assert (name != NULL);
+	g_return_val_if_fail (name != NULL, FALSE);
 	
 	if (eel_str_has_prefix (name, "/")) {
 		return FALSE;
@@ -221,7 +215,7 @@ preferences_preference_is_gconf_key (const char *name)
 static char *
 preferences_key_make (const char *name)
 {
-	g_assert (name != NULL);
+	g_return_val_if_fail (name != NULL, NULL);
 	
 	if (!preferences_preference_is_gconf_key (name)) {
 		return g_strdup (name);
@@ -240,7 +234,7 @@ preferences_get_default_value (const char *name)
 	PreferencesEntry *entry;
 	char *key;
 
-	g_assert (name != NULL);
+	g_return_val_if_fail (name != NULL, NULL);
 
 	key = preferences_key_make (name);
 
@@ -257,39 +251,13 @@ preferences_get_default_value (const char *name)
         return result;
 }
 
-static int
-preferences_callback_entry_compare (gconstpointer a,
-				    gconstpointer b)
-{
-	const PreferencesCallbackEntry *a_entry = a;
-	const PreferencesCallbackEntry *b_entry = b;
-
-	if (a_entry->callback < b_entry->callback) {
-		return -1;
-	}
-
-	if (a_entry->callback > b_entry->callback) {
-		return +1;
-	}
-
-	if (a_entry->callback_data < b_entry->callback_data) {
-		return -1;
-	}
-
-	if (a_entry->callback_data > b_entry->callback_data) {
-		return +1;
-	}
-
-	return 0;
-}
-
 /* Public preferences functions */
 
 gboolean
 eel_preferences_get_is_invisible (const char *name)
 {
-	g_assert (name != NULL);
-	g_assert (preferences_is_initialized ());
+	g_return_val_if_fail (name != NULL, FALSE);
+	g_return_val_if_fail (preferences_is_initialized (), FALSE);
 
 	return preferences_global_table_lookup_or_insert (name)->invisible;
 }
@@ -377,7 +345,7 @@ eel_preferences_get_enum (const char *name)
 	int               ret_val;
 	char             *str_value;
 	GConfValue       *value;
-	const EelEnumeration   *enumeration;
+	EelEnumeration   *enumeration;
 	PreferencesEntry *entry;
 
 	g_return_val_if_fail (name != NULL, 0);
@@ -398,6 +366,7 @@ eel_preferences_get_enum (const char *name)
 	if (value->type == GCONF_VALUE_INT) { /* compatibility path */
 		ret_val = preferences_gconf_value_get_int (value);
 		eel_gconf_value_free (value);
+		eel_enumeration_free (enumeration);
 		return ret_val;
 	}
 
@@ -409,8 +378,9 @@ eel_preferences_get_enum (const char *name)
 		return 0;
 	}
 
-	ret_val = eel_enumeration_get_value_for_name (enumeration, str_value);
-
+	ret_val = eel_enumeration_get_sub_value (enumeration, str_value);
+	eel_enumeration_free (enumeration);
+	
 	g_free (str_value);
 
 	return ret_val;
@@ -421,7 +391,7 @@ eel_preferences_set_enum (const char *name,
 			  int         int_value)
 {
 	const char       *str_value;
-	const EelEnumeration   *enumeration;
+	EelEnumeration   *enumeration;
 	PreferencesEntry *entry;
 
 	g_return_if_fail (name != NULL);
@@ -438,7 +408,7 @@ eel_preferences_set_enum (const char *name,
 		return;
 	}
 
-	str_value = eel_enumeration_get_name_for_value (enumeration, int_value);
+	str_value = eel_enumeration_get_sub_name (enumeration, int_value);
 
 	if (str_value == NULL) {
 		g_warning ("No enum match for '%d'", int_value);
@@ -485,63 +455,48 @@ eel_preferences_get (const char *name)
 }
 
 void
-eel_preferences_set_string_array (const char  *name,
-				  char       **strv_value)
+eel_preferences_set_string_list (const char *name,
+				 const EelStringList *string_list_value)
 {
-	GSList *slist;
-	int i;
 	char *key;
+	GSList *slist;
 
 	g_return_if_fail (name != NULL);
 	g_return_if_fail (preferences_is_initialized ());
 
-	slist = NULL;
-	if (strv_value != NULL) {
-		for (i = 0; strv_value[i] != NULL; i++) {
-			slist = g_slist_prepend (slist, strv_value[i]);
-		}
-		slist = g_slist_reverse (slist);
-	}
-
+	slist = eel_string_list_as_g_slist (string_list_value);
 	key = preferences_key_make (name);
 	eel_gconf_set_string_list (key, slist);
 	g_free (key);
 
-	g_slist_free (slist);
+	eel_g_slist_free_deep (slist);
 }
 
 static gboolean
-string_array_is_valid (char **strv, const char *enumeration_id)
+string_list_is_valid (const EelStringList *string_list,
+		      const char *enumeration_id)
 {
 	guint i;
-	gboolean res;
+	char *nth;
+	gboolean bad;
 
-	g_assert (strv != NULL);
-	g_assert (enumeration_id != NULL);
+	g_return_val_if_fail (string_list != NULL, FALSE);
+	g_return_val_if_fail (enumeration_id != NULL, FALSE);
 
-	res = TRUE;
-	for (i = 0; strv[i] != NULL; i++) {
-		const EelEnumeration *enumeration;
-
-		enumeration = eel_enumeration_lookup (enumeration_id);
-		if (!enumeration) {
-			res = FALSE;
-			break;
-		}
-
-		if (!eel_enumeration_contains_name (enumeration, strv[i])) {
-			res = FALSE;
-			break;
-		}
+	bad = FALSE;
+	for (i = 0; i < eel_string_list_get_length (string_list) && !bad; i++) {
+		nth = eel_string_list_nth (string_list, i);
+		bad = !eel_enumeration_id_contains_name (enumeration_id, nth);
+		g_free (nth);
 	}
 
-	return res;
+	return !bad;
 }
 
-char **
-eel_preferences_get_string_array (const char *name)
+EelStringList *
+eel_preferences_get_string_list (const char *name)
 {
- 	char **result;
+ 	EelStringList *result;
 	GConfValue *value;
 	PreferencesEntry *entry;
 	GConfValue *default_value;
@@ -550,9 +505,9 @@ eel_preferences_get_string_array (const char *name)
 	g_return_val_if_fail (preferences_is_initialized (), NULL);
 
 	value = preferences_get_value (name);
-	result = preferences_gconf_value_get_string_array (value);
+	result = preferences_gconf_value_get_string_list (value);
 	eel_gconf_value_free (value);
-
+	
 	entry = preferences_global_table_lookup_or_insert (name);
 	g_assert (entry != NULL);
 
@@ -562,22 +517,47 @@ eel_preferences_get_string_array (const char *name)
 	}
 
 	/* Do a sanity check on the validity of the values */
-	if (string_array_is_valid (result, entry->enumeration_id)) {
+	if (string_list_is_valid (result, entry->enumeration_id)) {
 		return result;
 	}
 
 	/* Forget the bad value and use the default instead */
-	g_strfreev (result);
+	eel_string_list_free (result);
 
 	default_value = preferences_get_default_value (name);
 	if (default_value) {		
-		result = preferences_gconf_value_get_string_array (default_value);
+		result = preferences_gconf_value_get_string_list (default_value);
 		gconf_value_free (default_value);
 	}
 
 	return result;
 }
 
+GList *
+eel_preferences_get_string_glist (const char *name)
+{
+	EelStringList *string_list;
+	GList *ret;
+	
+	string_list = eel_preferences_get_string_list (name);
+	ret = eel_string_list_as_g_list (string_list);
+	eel_string_list_free (string_list);
+
+	return ret;
+}
+
+void           
+eel_preferences_set_string_glist (const char *name,
+				  const GList *string_list_value)
+{
+	EelStringList *string_list;
+	
+	string_list = eel_string_list_new_from_g_list (string_list_value,
+						       TRUE);
+	eel_preferences_set_string_list (name, string_list);
+	eel_string_list_free (string_list);
+}
+
 void
 eel_preferences_unset (const char *name)
 {
@@ -623,7 +603,7 @@ preferences_callback_entry_invoke_function (gpointer data,
 {
 	PreferencesCallbackEntry *callback_entry;
 	
-	g_assert (data != NULL);
+	g_return_if_fail (data != NULL);
 	
 	callback_entry = data;
 
@@ -633,7 +613,7 @@ preferences_callback_entry_invoke_function (gpointer data,
 static void
 preferences_entry_invoke_callbacks (PreferencesEntry *entry)
 {
-	g_assert (entry != NULL);
+	g_return_if_fail (entry != NULL);
 
 	/* Update the auto storage preferences */
 	if (entry->auto_storage_list != NULL) {
@@ -654,8 +634,8 @@ update_auto_string (gpointer data, gpointer callback_data)
 	char **storage;
 	const char *value;
 
-	g_assert (data != NULL);
-	g_assert (callback_data != NULL);
+	g_return_if_fail (data != NULL);
+	g_return_if_fail (callback_data != NULL);
 
 	storage = (char **)data;
 	value = (const char *)callback_data;
@@ -665,49 +645,41 @@ update_auto_string (gpointer data, gpointer callback_data)
 }
 
 static void
-update_auto_string_array (gpointer data, gpointer callback_data)
+update_auto_string_list (gpointer data, gpointer callback_data)
 {
-	char ***storage;
-	char **value;
+	EelStringList **storage;
+	const EelStringList *value;
 	
-	g_assert (data != NULL);
-	g_assert (callback_data != NULL);
+	g_return_if_fail (data != NULL);
+	g_return_if_fail (callback_data != NULL);
 
-	storage = (char ***)data;
-	value = (char **)callback_data;
-
-	g_strfreev (*storage);
-	*(char ***)storage = value ? g_strdupv (value) : NULL;
+	storage = (EelStringList **)data;
+	value = (const EelStringList *)callback_data;
+	
+	eel_string_list_free (*storage);
+	*(EelStringList **)storage = eel_string_list_copy (value);
 }
 
 static void
-update_auto_string_array_as_quarks (gpointer data, gpointer callback_data)
+update_auto_string_glist (gpointer data, gpointer callback_data)
 {
-	GQuark **storage;
-	char **value;
-	int i = 0;
+	GList **storage;
+	GList *value;
 	
-	g_assert (data != NULL);
-	g_assert (callback_data != NULL);
+	g_return_if_fail (data != NULL);
+	g_return_if_fail (callback_data != NULL);
 
-	storage = (GQuark **)data;
-	value = (char **)callback_data;
-
-	g_free (*storage);
-	*storage = g_new (GQuark, g_strv_length (value) + 1);
-
-	if (value != NULL) {
-		for (i = 0; value[i] != NULL; ++i) {
-			(*storage)[i] = g_quark_from_string (value[i]);
-		}
-	}
-	(*storage)[i] = 0;
+	storage = (GList **)data;
+	value = (GList *)callback_data;
+	
+	eel_g_list_free_deep (*storage);
+	*(GList **)storage = eel_g_str_list_copy (value);
 }
 
 static void
 update_auto_integer_or_boolean (gpointer data, gpointer callback_data)
 {
-	g_assert (data != NULL);
+	g_return_if_fail (data != NULL);
 
 	*(int *)data = GPOINTER_TO_INT (callback_data);
 }
@@ -716,7 +688,8 @@ static void
 preferences_entry_update_auto_storage (PreferencesEntry *entry)
 {
 	char *new_string_value;
-	char **new_string_array_value;
+	EelStringList *new_string_list_value;
+	GList *new_string_glist_value;
 	int new_int_value;
 	gboolean new_boolean_value;
 
@@ -735,19 +708,19 @@ preferences_entry_update_auto_storage (PreferencesEntry *entry)
 			g_free (new_string_value);
 		}
 		break;
-	case PREFERENCE_STRING_ARRAY:
-		new_string_array_value = eel_preferences_get_string_array (entry->name);
+	case PREFERENCE_STRING_LIST:
+		new_string_list_value = eel_preferences_get_string_list (entry->name);
 		g_list_foreach (entry->auto_storage_list,
-				update_auto_string_array,
-				new_string_array_value);
-		g_strfreev (new_string_array_value);
+				update_auto_string_list,
+				new_string_list_value);
+		eel_string_list_free (new_string_list_value);
 		break;
-	case PREFERENCE_STRING_ARRAY_AS_QUARKS:
-		new_string_array_value = eel_preferences_get_string_array (entry->name);
+	case PREFERENCE_STRING_GLIST:
+		new_string_glist_value = eel_preferences_get_string_glist (entry->name);
 		g_list_foreach (entry->auto_storage_list,
-				update_auto_string_array_as_quarks,
-				new_string_array_value);
-		g_strfreev (new_string_array_value);
+				update_auto_string_glist,
+				new_string_glist_value);
+		eel_g_list_free_deep (new_string_glist_value);
 		break;
 	case PREFERENCE_INTEGER:
 		new_int_value = eel_preferences_get_integer (entry->name);
@@ -772,9 +745,9 @@ preferences_something_changed_notice (GConfClient *client,
 				      GConfEntry *entry, 
 				      gpointer notice_data)
 {
-	g_assert (entry != NULL);
-	g_assert (entry->key != NULL);
-	g_assert (notice_data != NULL);
+	g_return_if_fail (entry != NULL);
+	g_return_if_fail (entry->key != NULL);
+	g_return_if_fail (notice_data != NULL);
 
 	preferences_entry_invoke_callbacks (notice_data);
 }
@@ -793,7 +766,7 @@ preferences_entry_ensure_gconf_connection (PreferencesEntry *entry)
 		return;
 	}
 		
-	g_assert (entry->name != NULL);
+	g_return_if_fail (entry->name != NULL);
 
 	key = preferences_key_make (entry->name);
 
@@ -802,7 +775,7 @@ preferences_entry_ensure_gconf_connection (PreferencesEntry *entry)
 								 entry);
 	g_free (key);
 
-	g_assert (entry->gconf_connection_id != EEL_GCONF_UNDEFINED_CONNECTION);
+	g_return_if_fail (entry->gconf_connection_id != EEL_GCONF_UNDEFINED_CONNECTION);
 }
 
 /**
@@ -820,22 +793,19 @@ preferences_entry_add_callback (PreferencesEntry *entry,
 				gpointer callback_data)
 {
 	PreferencesCallbackEntry *callback_entry;
-	GList *l;
 
-	g_assert (entry != NULL);
-	g_assert (callback != NULL);
+	g_return_if_fail (entry != NULL);
+	g_return_if_fail (callback != NULL);
 
 	callback_entry = g_new0 (PreferencesCallbackEntry, 1);
 	callback_entry->callback = callback;
 	callback_entry->callback_data = callback_data;
+	
+	g_return_if_fail (callback_entry != NULL);
+	
+	entry->callback_list = g_list_append (entry->callback_list, callback_entry);
 
-	l = g_list_find_custom (entry->callback_list, callback_entry, preferences_callback_entry_compare);
-	if (l == NULL) {
-		entry->callback_list = g_list_append (entry->callback_list, callback_entry);
-		preferences_entry_ensure_gconf_connection (entry);
-	} else {
-		g_warning ("Trying to add a callback for %s that already exists.", entry->name);
-	}
+	preferences_entry_ensure_gconf_connection (entry);
 }
 
 /**
@@ -853,13 +823,10 @@ preferences_entry_add_auto_storage (PreferencesEntry *entry,
 				    gpointer storage,
 				    PreferenceType type)
 {
-	g_assert (entry != NULL);
-	g_assert (storage != NULL);
-	g_assert (entry->type == 0 || entry->type == type);
-	if (g_list_find (entry->auto_storage_list, storage) != NULL) {
-		g_warning ("Trying to add an auto storage for %s that already exists.", entry->name);
-		return;
-	}
+	g_return_if_fail (entry != NULL);
+	g_return_if_fail (storage != NULL);
+	g_return_if_fail (entry->type == 0 || entry->type == type);
+	g_return_if_fail (g_list_find (entry->auto_storage_list, storage) == NULL);
 
 	entry->type = type;
 	
@@ -897,25 +864,32 @@ preferences_entry_remove_callback (PreferencesEntry *entry,
 				   EelPreferencesCallback callback,
 				   gpointer callback_data)
 {
-	PreferencesCallbackEntry cb_entry;
-	GList *l;
+	GList *new_list;
+	const GList *node;
 
-	g_assert (entry != NULL);
-	g_assert (callback != NULL);
-
-	cb_entry.callback = callback;
-	cb_entry.callback_data = callback_data;
-
-	l = g_list_find_custom (entry->callback_list, &cb_entry, preferences_callback_entry_compare);
-	if (l != NULL) {
-		preferences_callback_entry_free (l->data);
-		entry->callback_list = g_list_delete_link (entry->callback_list, l);
-		preferences_entry_check_remove_connection (entry);
-	} else {
-		g_warning ("Trying to remove a callback for %s without adding it first.", entry->name);
+	g_return_if_fail (entry != NULL);
+	g_return_if_fail (callback != NULL);
+	g_return_if_fail (entry->callback_list != NULL);
+	
+	new_list = g_list_copy (entry->callback_list);
+	
+	for (node = new_list; node != NULL; node = node->next) {
+		PreferencesCallbackEntry *callback_entry = node->data;
+		
+		g_return_if_fail (callback_entry != NULL);
+		
+		if (callback_entry->callback == callback &&
+		    callback_entry->callback_data == callback_data) {
+			entry->callback_list = g_list_remove (entry->callback_list, 
+							      callback_entry);
+			
+			preferences_callback_entry_free (callback_entry);
+		}
 	}
 
-	g_assert (g_list_find_custom (entry->callback_list, &cb_entry, preferences_callback_entry_compare) == NULL);
+	g_list_free (new_list);
+
+	preferences_entry_check_remove_connection (entry);
 }
 
 /**
@@ -933,16 +907,16 @@ preferences_entry_remove_auto_storage (PreferencesEntry *entry,
 	const GList *node;
 	gpointer storage_in_entry;
 
-	g_assert (entry != NULL);
-	g_assert (storage != NULL);
-	g_assert (entry->auto_storage_list != NULL);
+	g_return_if_fail (entry != NULL);
+	g_return_if_fail (storage != NULL);
+	g_return_if_fail (entry->auto_storage_list != NULL);
 	
 	new_list = g_list_copy (entry->auto_storage_list);
 	
 	for (node = new_list; node != NULL; node = node->next) {
 		storage_in_entry = node->data;
 		
-		g_assert (storage_in_entry != NULL);
+		g_return_if_fail (storage_in_entry != NULL);
 		
 		if (storage_in_entry == storage) {
 			entry->auto_storage_list = g_list_remove (entry->auto_storage_list, 
@@ -952,15 +926,12 @@ preferences_entry_remove_auto_storage (PreferencesEntry *entry,
 			case PREFERENCE_STRING:
 				update_auto_string (storage, NULL);
 				break;
-			case PREFERENCE_STRING_ARRAY:
-				update_auto_string_array (storage, NULL);
-				break;
-			case PREFERENCE_STRING_ARRAY_AS_QUARKS:
-				update_auto_string_array_as_quarks (storage, NULL);
+			case PREFERENCE_STRING_LIST:
+				update_auto_string_list (storage, NULL);
 				break;
 			case PREFERENCE_BOOLEAN:
 			case PREFERENCE_INTEGER:
-				update_auto_integer_or_boolean (storage, NULL);
+				update_auto_integer_or_boolean (storage, 0);
 				break;
 			default:
 				g_warning ("unexpected preference type %d in preferences_entry_remove_auto_storage", entry->type);
@@ -982,7 +953,7 @@ preferences_entry_remove_auto_storage (PreferencesEntry *entry,
 static void
 preferences_callback_entry_free (PreferencesCallbackEntry *callback_entry)
 {
-	g_assert (callback_entry != NULL);
+	g_return_if_fail (callback_entry != NULL);
 
 	callback_entry->callback = NULL;
 	callback_entry->callback_data = NULL;
@@ -1002,7 +973,7 @@ static void
 preferences_callback_entry_free_func (gpointer	data,
 				      gpointer	callback_data)
 {
-	g_assert (data != NULL);
+	g_return_if_fail (data != NULL);
 	
 	preferences_callback_entry_free (data);
 }
@@ -1016,7 +987,7 @@ preferences_callback_entry_free_func (gpointer	data,
 static void
 preferences_entry_free (PreferencesEntry *entry)
 {
-	g_assert (entry != NULL);
+	g_return_if_fail (entry != NULL);
 
 	eel_gconf_notification_remove (entry->gconf_connection_id);
 	entry->gconf_connection_id = EEL_GCONF_UNDEFINED_CONNECTION;
@@ -1072,12 +1043,6 @@ preferences_global_table_free (void)
 	storage_path = NULL;
 }
 
-static void
-preferences_uninitialize (void)
-{
-	initialized = FALSE;
-}
-
 static GHashTable *
 preferences_global_table_get_global (void)
 {
@@ -1089,8 +1054,6 @@ preferences_global_table_get_global (void)
 		if (!at_exit_handler_added) {
 			at_exit_handler_added = TRUE;
 			eel_debug_call_at_shutdown (preferences_global_table_free);
-			/* ensure that we catch calls to preferences functions after eel shutdown */
-			eel_debug_call_at_shutdown (preferences_uninitialize);
 		}
 	}
 	
@@ -1100,8 +1063,8 @@ preferences_global_table_get_global (void)
 static PreferencesEntry *
 preferences_global_table_lookup (const char *name)
 {
-	g_assert (name != NULL);
-	g_assert (preferences_global_table_get_global () != NULL);
+	g_return_val_if_fail (name != NULL, NULL);
+	g_return_val_if_fail (preferences_global_table_get_global () != NULL, NULL);
 	
 	return g_hash_table_lookup (preferences_global_table_get_global (), name);
 }
@@ -1111,16 +1074,16 @@ preferences_global_table_insert (const char *name)
 {
 	PreferencesEntry *entry;
 
-	g_assert (name != NULL);
-	g_assert (preferences_global_table_get_global () != NULL);
-	g_assert (preferences_global_table_lookup (name) == NULL);
+	g_return_val_if_fail (name != NULL, NULL);
+	g_return_val_if_fail (preferences_global_table_get_global () != NULL, NULL);
+	g_return_val_if_fail (preferences_global_table_lookup (name) == NULL, NULL);
 	
 	entry = g_new0 (PreferencesEntry, 1);
 	entry->name = g_strdup (name);
 
 	g_hash_table_insert (preferences_global_table_get_global (), entry->name, entry);
 
-	g_assert (entry == preferences_global_table_lookup (name));
+	g_return_val_if_fail (entry == preferences_global_table_lookup (name), NULL);
 
 	return entry;
 }
@@ -1130,7 +1093,7 @@ preferences_global_table_lookup_or_insert (const char *name)
 {
 	PreferencesEntry *entry;
 
-	g_assert (name != NULL);
+	g_return_val_if_fail (name != NULL, NULL);
 	
 	entry = preferences_global_table_lookup (name);
 
@@ -1183,11 +1146,11 @@ eel_preferences_add_auto_string (const char *name,
 }
 
 void
-eel_preferences_add_auto_string_array (const char   *name,
-				       char       ***storage)
+eel_preferences_add_auto_string_list (const char *name,
+				      const EelStringList **storage)
 {
 	PreferencesEntry *entry;
-	char **value;
+	EelStringList *value;
 
 	g_return_if_fail (name != NULL);
 	g_return_if_fail (storage != NULL);
@@ -1196,19 +1159,19 @@ eel_preferences_add_auto_string_array (const char   *name,
 	entry = preferences_global_table_lookup_or_insert (name);
 	g_assert (entry != NULL);
 
-	preferences_entry_add_auto_storage (entry, storage, PREFERENCE_STRING_ARRAY);
+	preferences_entry_add_auto_storage (entry, storage, PREFERENCE_STRING_LIST);
 
-	value = eel_preferences_get_string_array (entry->name);
-	update_auto_string_array (storage, value);
-	g_strfreev (value);
+	value = eel_preferences_get_string_list (entry->name);
+	update_auto_string_list (storage, value);
+	eel_string_list_free (value);
 }
 
 void
-eel_preferences_add_auto_string_array_as_quarks (const char *name,
-						 GQuark **storage)
+eel_preferences_add_auto_string_glist (const char *name,
+				       const GList **storage)
 {
 	PreferencesEntry *entry;
-	char **value;
+	GList *value;
 
 	g_return_if_fail (name != NULL);
 	g_return_if_fail (storage != NULL);
@@ -1217,11 +1180,11 @@ eel_preferences_add_auto_string_array_as_quarks (const char *name,
 	entry = preferences_global_table_lookup_or_insert (name);
 	g_assert (entry != NULL);
 
-	preferences_entry_add_auto_storage (entry, storage, PREFERENCE_STRING_ARRAY_AS_QUARKS);
+	preferences_entry_add_auto_storage (entry, storage, PREFERENCE_STRING_GLIST);
 
-	value = eel_preferences_get_string_array (entry->name);
-	update_auto_string_array_as_quarks (storage, value);
-	g_free (value);
+	value = eel_preferences_get_string_glist (entry->name);
+	update_auto_string_glist (storage, value);
+	eel_g_list_free_deep (value);
 }
 
 void
@@ -1306,8 +1269,8 @@ eel_preferences_remove_auto_string (const char *name,
 }
 
 void
-eel_preferences_remove_auto_string_array (const char *name,
-					  char ***storage)
+eel_preferences_remove_auto_string_list (const char *name,
+					 const EelStringList **storage)
 {
 	PreferencesEntry *entry;
 
@@ -1378,13 +1341,9 @@ preferences_while_alive_disconnector (gpointer callback_data, GObject *where_obj
 
 	data = callback_data;
 
-	/* we might have survived an eel shutdown, which
-	 * already cleared all the callbacks */
-	if (preferences_is_initialized ()) {
-		eel_preferences_remove_callback (data->name,
-						 data->callback,
-						 data->callback_data);
-	}
+	eel_preferences_remove_callback (data->name,
+					 data->callback,
+					 data->callback_data);
 
 	g_free (data->name);
 	g_free (data);
@@ -1502,8 +1461,8 @@ preferences_set_emergency_fallback_stealing_value (const char *name,
 {
 	PreferencesEntry *entry;
 
-	g_assert (name != NULL);
-	g_assert (preferences_is_initialized ());
+	g_return_if_fail (name != NULL);
+	g_return_if_fail (preferences_is_initialized ());
 
 	entry = preferences_global_table_lookup_or_insert (name);
 	g_assert (entry != NULL);
@@ -1559,32 +1518,34 @@ eel_preferences_set_emergency_fallback_boolean (const char *name,
 	preferences_set_emergency_fallback_stealing_value (name, gconf_value);
 }
 
+static void
+listify_strings_foreach (const char *string,
+			 gpointer callback_data)
+{
+	GSList **listp = callback_data;
+	GConfValue *value;
+
+	value = gconf_value_new (GCONF_VALUE_STRING);
+	gconf_value_set_string (value, string);
+	
+	*listp = g_slist_prepend (*listp, value);
+}
 
 void
-eel_preferences_set_emergency_fallback_string_array (const char  *name,
-						     char       **value)
+eel_preferences_set_emergency_fallback_string_list (const char    *name,
+						    EelStringList *value)
 {
 	GConfValue *gconf_value;
 	GSList *list;
-	int i;
 
 	g_return_if_fail (name != NULL);
 	g_return_if_fail (value != NULL);
-
+	
 	gconf_value = gconf_value_new (GCONF_VALUE_LIST);
+
 	gconf_value_set_list_type (gconf_value, GCONF_VALUE_STRING);
-
 	list = NULL;
-	for (i = 0; value[i] != NULL; ++i)
-	{
-		GConfValue *v;
-
-		v = gconf_value_new (GCONF_VALUE_STRING);
-		gconf_value_set_string (v, value[i]);
-
-		list = g_slist_prepend (list, v);
-	}
-
+	eel_string_list_for_each (value, listify_strings_foreach, &list);
 	gconf_value_set_list_nocopy (gconf_value, g_slist_reverse (list));
 
 	preferences_set_emergency_fallback_stealing_value (name, gconf_value);
